@@ -28,7 +28,7 @@ import math
 import sys
 import struct
 from Crypto import Random
-from Crypto.Util.py3compat import _memoryview, iter_range
+from Crypto.Util.py3compat import iter_range
 
 # Backward compatibility
 _fastmath = None
@@ -51,12 +51,8 @@ def size (N):
     """Returns the size of the number N in bits."""
 
     if N < 0:
-        raise ValueError("Size in bits only avialable for non-negative numbers")
-
-    bits = 0
-    while N >> bits:
-        bits += 1
-    return bits
+        raise ValueError("Size in bits only available for non-negative numbers")
+    return N.bit_length()
 
 
 def getRandomInteger(N, randfunc=None):
@@ -113,27 +109,56 @@ def getRandomNBitInteger(N, randfunc=None):
     assert size(value) >= N
     return value
 
-def GCD(x,y):
-    """Greatest Common Denominator of :data:`x` and :data:`y`.
-    """
 
-    x = abs(x) ; y = abs(y)
-    while x > 0:
-        x, y = y % x, x
-    return y
+if sys.version_info[:2] >= (3, 5):
 
-def inverse(u, v):
-    """The inverse of :data:`u` *mod* :data:`v`."""
+    GCD = math.gcd
 
-    u3, v3 = u, v
-    u1, v1 = 1, 0
-    while v3 > 0:
-        q = u3 // v3
-        u1, v1 = v1, u1 - v1*q
-        u3, v3 = v3, u3 - v3*q
-    while u1<0:
-        u1 = u1 + v
-    return u1
+else:
+
+    def GCD(x,y):
+        """Greatest Common Denominator of :data:`x` and :data:`y`.
+        """
+
+        x = abs(x) ; y = abs(y)
+        while x > 0:
+            x, y = y % x, x
+        return y
+
+
+if sys.version_info[:2] >= (3, 8):
+
+    def inverse(u, v):
+        """The inverse of :data:`u` *mod* :data:`v`."""
+
+        if v == 0:
+            raise ZeroDivisionError("Modulus cannot be zero")
+        if v < 0:
+            raise ValueError("Modulus cannot be negative")
+
+        return pow(u, -1, v)
+
+else:
+
+    def inverse(u, v):
+        """The inverse of :data:`u` *mod* :data:`v`."""
+
+        if v == 0:
+            raise ZeroDivisionError("Modulus cannot be zero")
+        if v < 0:
+            raise ValueError("Modulus cannot be negative")
+
+        u3, v3 = u, v
+        u1, v1 = 1, 0
+        while v3 > 0:
+            q = u3 // v3
+            u1, v1 = v1, u1 - v1*q
+            u3, v3 = v3, u3 - v3*q
+        if u3 != 1:
+            raise ValueError("No inverse value can be computed")
+        while u1<0:
+            u1 = u1 + v
+        return u1
 
 # Given a number of bits to generate and a random generation function,
 # find a prime number of the appropriate size.
@@ -141,14 +166,19 @@ def inverse(u, v):
 def getPrime(N, randfunc=None):
     """Return a random N-bit prime number.
 
+    N must be an integer larger than 1.
     If randfunc is omitted, then :meth:`Random.get_random_bytes` is used.
     """
     if randfunc is None:
         randfunc = Random.get_random_bytes
 
-    number=getRandomNBitInteger(N, randfunc) | 1
-    while (not isPrime(number, randfunc=randfunc)):
-        number=number+2
+    if N < 2:
+        raise ValueError("N must be larger than 1")
+
+    while True:
+        number = getRandomNBitInteger(N, randfunc) | 1
+        if isPrime(number, randfunc=randfunc):
+            break
     return number
 
 
@@ -254,7 +284,7 @@ def getStrongPrime(N, e=0, false_positive_prob=1e-6, randfunc=None):
     # calculate range for X
     #   lower_bound = sqrt(2) * 2^{511 + 128*x}
     #   upper_bound = 2^{512 + 128*x} - 1
-    x = (N - 512) >> 7;
+    x = (N - 512) >> 7
     # We need to approximate the sqrt(2) in the lower_bound by an integer
     # expression because floating point math overflows with these numbers
     lower_bound = (14142135623730950489 * (2 ** (511 + 128*x))) //  10000000000000000000
@@ -348,7 +378,7 @@ def isPrime(N, false_positive_prob=1e-6, randfunc=None):
           If omitted, :func:`Crypto.Random.get_random_bytes` is used.
 
     Return:
-        `True` is the input is indeed prime.
+        `True` if the input is indeed prime.
     """
 
     if randfunc is None:
@@ -361,12 +391,12 @@ def isPrime(N, false_positive_prob=1e-6, randfunc=None):
         return N == 2
     for p in sieve_base:
         if N == p:
-            return 1
+            return True
         if N % p == 0:
-            return 0
+            return False
 
     rounds = int(math.ceil(-math.log(false_positive_prob)/math.log(4)))
-    return _rabinMillerTest(N, rounds, randfunc)
+    return bool(_rabinMillerTest(N, rounds, randfunc))
 
 
 # Improved conversion functions contributed by Barry Warsaw, after
@@ -375,46 +405,72 @@ def isPrime(N, false_positive_prob=1e-6, randfunc=None):
 import struct
 
 def long_to_bytes(n, blocksize=0):
-    """Convert an integer to a byte string.
+    """Convert a positive integer to a byte string using big endian encoding.
 
-    In Python 3.2+, use the native method instead::
-
-        >>> n.to_bytes(blocksize, 'big')
-
-    For instance::
-
-        >>> n = 80
-        >>> n.to_bytes(2, 'big')
-        b'\x00P'
-
-    If the optional :data:`blocksize` is provided and greater than zero,
-    the byte string is padded with binary zeros (on the front) so that
-    the total length of the output is a multiple of blocksize.
-
-    If :data:`blocksize` is zero or not provided, the byte string will
+    If :data:`blocksize` is absent or zero, the byte string will
     be of minimal length.
+
+    Otherwise, the length of the byte string is guaranteed to be a multiple
+    of :data:`blocksize`. If necessary, zeroes (``\\x00``) are added at the left.
+
+    .. note::
+        In Python 3, if you are sure that :data:`n` can fit into
+        :data:`blocksize` bytes, you can simply use the native method instead::
+
+            >>> n.to_bytes(blocksize, 'big')
+
+        For instance::
+
+            >>> n = 80
+            >>> n.to_bytes(2, 'big')
+            b'\\x00P'
+
+        However, and unlike this ``long_to_bytes()`` function,
+        an ``OverflowError`` exception is raised if :data:`n` does not fit.
     """
-    # after much testing, this algorithm was deemed to be the fastest
-    s = b''
-    n = int(n)
+
+    if n < 0 or blocksize < 0:
+        raise ValueError("Values must be non-negative")
+
+    result = []
     pack = struct.pack
-    while n > 0:
-        s = pack('>I', n & 0xffffffff) + s
+
+    # Fill the first block independently from the value of n
+    bsr = blocksize
+    while bsr >= 8:
+        result.insert(0, pack('>Q', n & 0xFFFFFFFFFFFFFFFF))
+        n = n >> 64
+        bsr -= 8
+
+    while bsr >= 4:
+        result.insert(0, pack('>I', n & 0xFFFFFFFF))
         n = n >> 32
-    # strip off leading zeros
-    for i in range(len(s)):
-        if s[i] != b'\x00'[0]:
-            break
+        bsr -= 4
+
+    while bsr > 0:
+        result.insert(0, pack('>B', n & 0xFF))
+        n = n >> 8
+        bsr -= 1
+
+    if n == 0:
+        if len(result) == 0:
+            bresult = b'\x00'
+        else:
+            bresult = b''.join(result)
     else:
-        # only happens when n == 0
-        s = b'\x00'
-        i = 0
-    s = s[i:]
-    # add back some pad bytes.  this could be done more efficiently w.r.t. the
-    # de-padding being done above, but sigh...
-    if blocksize > 0 and len(s) % blocksize:
-        s = (blocksize - len(s) % blocksize) * b'\x00' + s
-    return s
+        # The encoded number exceeds the block size
+        while n > 0:
+            result.insert(0, pack('>Q', n & 0xFFFFFFFFFFFFFFFF))
+            n = n >> 64
+        result[0] = result[0].lstrip(b'\x00')
+        bresult = b''.join(result)
+        # bresult has minimum length here
+        if blocksize > 0:
+            target_len = ((len(bresult) - 1) // blocksize + 1) * blocksize
+            bresult = b'\x00' * (target_len - len(bresult)) + bresult
+
+    return bresult
+
 
 def bytes_to_long(s):
     """Convert a byte string to a long integer (big endian).
@@ -439,7 +495,7 @@ def bytes_to_long(s):
     if sys.version_info[0:3] < (2, 7, 4):
         if isinstance(s, bytearray):
             s = bytes(s)
-        elif isinstance(s, _memoryview):
+        elif isinstance(s, memoryview):
             s = s.tobytes()
 
     length = len(s)
